﻿[<ReflectedDefinition>]
module Maze

open Window

let maze = "\
##/------------7/------------7##,\
##|............|!............|##,\
##|./__7./___7.|!./___7./__7.|##,\
##|o|  !.|   !.|!.|   !.|  !o|##,\
##|.L--J.L---J.LJ.L---J.L--J.|##,\
##|..........................|##,\
##|./__7./7./______7./7./__7.|##,\
##|.L--J.|!.L--7/--J.|!.L--J.|##,\
##|......|!....|!....|!......|##,\
##L____7.|L__7 |! /__J!./____J##,\
#######!.|/--J LJ L--7!.|#######,\
#######!.|!          |!.|#######,\
#######!.|! /__==__7 |!.|#######,\
-------J.LJ |      ! LJ.L-------,\
########.   | **** !   .########,\
_______7./7 |      ! /7./_______,\
#######!.|! L______J |!.|#######,\
#######!.|!          |!.|#######,\
#######!.|! /______7 |!.|#######,\
##/----J.LJ L--7/--J LJ.L----7##,\
##|............|!............|##,\
##|./__7./___7.|!./___7./__7.|##,\
##|.L-7!.L---J.LJ.L---J.|/-J.|##,\
##|o..|!.......<>.......|!..o|##,\
##L_7.|!./7./______7./7.|!./_J##,\
##/-J.LJ.|!.L--7/--J.|!.LJ.L-7##,\
##|......|!....|!....|!......|##,\
##|./____JL__7.|!./__JL____7.|##,\
##|.L--------J.LJ.L--------J.|##,\
##|..........................|##,\
##L--------------------------J##".Split(',')

let tileChars = "/_7|!L-J=.o"
let tileFiles = [|"tl";"top";"tr";"left";"right";"bl";"bottom";"br";"top";"dot";"pill"|]

let tile t image =
    let context = context()
    for y = 0 to maze.Length-1 do
        let line = maze.[y]
        for x = 0 to line.Length-1 do
            let c = line.[x]
            if c = t then context.drawImage(image, float (x*8), float (y*8))

let renderMaze () =
    (0.,0.,256.,256.) |> filled (rgb 0 0 0) 
    for i = 0 to tileChars.Length-1 do
        let t = tileChars.[i]
        let image = unbox<ts.HTMLImageElement>(ts.document.createElement("image"))   
        image.src <- tileFiles.[i] + ".png"
        let image = image.AsHTMLElement()
        image.onload <- (fun e -> tile t image; null)

let isWall (c:char) = "_|!/7LJ-".IndexOf(c) <> -1

let tileAt (x,y) = if x < 0 || x > 30 then ' ' else maze.[y].[x]
        
let isWallAt (x,y) = tileAt(x,y) |> isWall

let noWall (x,y) (ex,ey) =
    let bx, by = (x+6+ex) >>> 3, (y+6+ey) >>> 3
    isWallAt (bx,by) |> not

let verticallyAligned (x,y) =  (x % 8) = 5
let horizontallyAligned (x,y) = (y % 8) = 5
let isAligned n = (n % 8) = 5

let canGoUp (x,y) = isAligned x && noWall (x,y) (0,-4)
let canGoDown (x,y) = isAligned x && noWall (x,y) (0,5)
let canGoLeft (x,y) = isAligned y && noWall (x,y) (-4,0)
let canGoRight (x,y) = isAligned y && noWall (x,y) (5,0)

